
function rltext.RGBAToHex( r, g, b, a )
	r = (r>1 and 1) or (r<0 and 0) or r
	g = (g>1 and 1) or (g<0 and 0) or g
	b = (b>1 and 1) or (b<0 and 0) or b
	return string.format("%02x%02x%02x%02x", r * 255, g * 255, b * 255, a * 255)
end

function rltext.HexToRGBA( hex )
	local rhex, ghex, bhex, ahex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6), string.sub(hex, 7, 8)
	return tonumber(rhex, 16)/255, tonumber(ghex, 16)/255, tonumber(bhex, 16)/255
end


function rltext:LibAddonInit()
	local LAM = LibStub('LibAddonMenu-2.0')
	if LAM then
		local LMP = LibStub("LibMediaProvider-1.0")
		if LMP then
			local panelData = {
				type = 'panel',
				name = "RaidLead |c710000Essentials|r",
				displayName = "RaidLead |c710000Essentials|r",
				author = "|cfd4bf4@LaynaKaru",
				version = self.version,
				registerForRefresh = true,
			}
			LAM:RegisterAddonPanel('RaidLeadTextOptions', panelData)

			local optionsData = {}

			local screenWidth, screenHeight = GuiRoot:GetDimensions()

			table.insert(optionsData, {
				type = 'header',
				name = "General Settings",
				width = "full",
			})
			table.insert(optionsData, {
				type = 'button',
				name = "Preview",
				tooltip = "Yay!",
				func = function()
					if rltext.bIsPositioning then return end

					rltext.bIsPreview = not(rltext.bIsPreview)
					if rltext.bIsPreview then
						rltext.Preview()
					else
						rltext.stopPreviews()
					end
				end,
				width = "half",
			})

			table.insert(optionsData, {
				type = 'button',
				name = "Position Frames",
				tooltip = "Yay!",
				func = function()
					rltext.bIsPositioning = not(rltext.bIsPositioning)
					rltext.bIsPreview = not(rltext.bIsPreview)

					if rltext.bIsPositioning then
						rltext.Preview()
						RLTEXT_OVERLAY:SetMouseEnabled(true)
						RLTEXT_OVERLAYSEC:SetMouseEnabled(true)
						RLTEXT_OVERLAYMEM:SetMouseEnabled(true)
						RLTEXT_OVERLAY:SetMovable(true)
						RLTEXT_OVERLAY_BG:SetAlpha(0.5)
						RLTEXT_OVERLAYSEC:SetMovable(true)
						RLTEXT_OVERLAYSEC_BG:SetAlpha(0.5)
						RLTEXT_OVERLAYMEM:SetMovable(true)
						RLTEXT_OVERLAYMEM_BG:SetAlpha(0.5)
					else
						rltext.stopPreviews()
						RLTEXT_OVERLAY:SetMouseEnabled(false)
						RLTEXT_OVERLAYSEC:SetMouseEnabled(false)
						RLTEXT_OVERLAYMEM:SetMouseEnabled(false)
						RLTEXT_OVERLAY:SetMovable(false)
						RLTEXT_OVERLAY_BG:SetAlpha(0)
						RLTEXT_OVERLAYSEC:SetMovable(false)
						RLTEXT_OVERLAYSEC_BG:SetAlpha(0)
						RLTEXT_OVERLAYMEM:SetMovable(false)
						RLTEXT_OVERLAYMEM_BG:SetAlpha(0)
					end
				end,
				width = "half",
			})

		--[[
			table.insert(optionsData, {
				type = "button",
				name = "Reload UI",
				tooltip = "Reloads your UI",
				func = function() ReloadUI() end,
				width = "half",
			})
		--]]
			table.insert(optionsData, {
				type = 'checkbox',
				name = "Disable",
				tooltip = "Can also disable by typing \"/rle off\" and re-enable by typing \"/rle on\"",
				getFunc = function() return self:ActiveSettings().disabled end,
				setFunc = function(value)
						self:ActiveSettings().disabled = value
						rltext.HideCheck()
					end,
				width = "half",
			})
			table.insert(optionsData, {
				type = 'checkbox',
				name = "Enable Secondary Lead Saving",
				tooltip = "Enables saving of set Secondary Leaders across groups",
				getFunc = function() return self:ActiveSettings().savesec end,
				setFunc = function(value) self:ActiveSettings().savesec = value end,
				width = "half",
			})
			table.insert(optionsData, {
				type = 'checkbox',
				name = "Disable #invite",
				tooltip = "Secondary Leaders can invite by typing \"#invite playername\", this disables that feature",
				getFunc = function() return self:ActiveSettings().disabledinv end,
				setFunc = function(value) self:ActiveSettings().disabledinv = value end,
				width = "half",
			})
			table.insert(optionsData, {
				type = 'checkbox',
				name = "Disable #kick",
				tooltip = "Secondary Leaders can kick by typing \"#kick playername\", this disables that feature",
				getFunc = function() return self:ActiveSettings().disabledkik end,
				setFunc = function(value) self:ActiveSettings().disabledkik = value end,
				width = "half",
			})
			table.insert(optionsData, {
				type = 'header',
				name = "Text Settings",
			})
			table.insert(optionsData, {
				type = 'slider',
				name = "Text Time",
				tooltip = "How long the text stays on the screen for",
				min = 1,
				max = 20,
				getFunc = function() return self:ActiveSettings().ttime end,
				setFunc = function(value) self:ActiveSettings().ttime = value end,
			})
			table.insert(optionsData, {
				type = 'slider',
				name = "Wrapping",
				tooltip = "Determines how many characters are needed before wrapping occours",
				min = 20,
				max = 100,
				getFunc = function() return self:ActiveSettings().wrapChars end,
				setFunc = function(value) self:ActiveSettings().wrapChars = value end,
			})

			table.insert(optionsData, {
				type = "fontblock",
				name = "Group Leader Font",
				getFace = function() return self:ActiveSettings().font end,
				getSize = function() return self:ActiveSettings().fontsize end,
				getOutline = function() return self:ActiveSettings().outline end,
				getColor = function() 
					local r, g, b, a = rltext.HexToRGBA(self:ActiveSettings().fontcolor)
					return r,g,b
				end,
				setFace = function(font)
					self:ActiveSettings().font = font
					rltext:OnFontChanged()
				end,
				setSize = function(size)
					self:ActiveSettings().fontsize = size
					rltext:OnFontChanged()
				end,
				setOutline = function(outline)
					self:ActiveSettings().outline = outline
					rltext:OnFontChanged()
				end,
				setColor = function(r, g, b)
					self:ActiveSettings().fontcolor = rltext.RGBAToHex(r, g, b, 1)
					rltext:OnFontChanged()
					if rltext.bIsPreview then
						rltext.Preview()
					end
				end,
			})

			table.insert(optionsData, {
				type = 'colorpicker',
				name = "Group Leader Name Color",
				tooltip = "",
				getFunc = function()
					local r, g, b, a = rltext.HexToRGBA(self:ActiveSettings().namecolor)
					return r,g,b
				end,
				setFunc = function(r,g,b)
					self:ActiveSettings().namecolor = rltext.RGBAToHex(r, g, b, 1)
					rltext:OnFontChanged()
					if rltext.bIsPreview then
						rltext.Preview()
					end
				end,
			})


			table.insert(optionsData, {
				type = "fontblock",
				name = "Secondary Lead Font",
				getFace = function() return self:ActiveSettings().slfont end,
				getSize = function() return self:ActiveSettings().slfontsize end,
				getOutline = function() return self:ActiveSettings().sloutline end,
				getColor = function() 
					local r, g, b, a = rltext.HexToRGBA(self:ActiveSettings().slfontcolor)
					return r,g,b
				end,
				setFace = function(font)
					self:ActiveSettings().slfont = font
					rltext:OnFontChanged()
				end,
				setSize = function(size)
					self:ActiveSettings().slfontsize = size
					rltext:OnFontChanged()
				end,
				setOutline = function(outline)
					self:ActiveSettings().sloutline = outline
					rltext:OnFontChanged()
				end,
				setColor = function(r, g, b)
					self:ActiveSettings().slfontcolor = rltext.RGBAToHex(r, g, b, 1)
					rltext:OnFontChanged()
					if rltext.bIsPreview then
						rltext.Preview()
					end
				end,
			})

			table.insert(optionsData, {
				type = 'colorpicker',
				name = "Secondary Lead Name Color",
				tooltip = "",
				getFunc = function()
					local r, g, b, a = rltext.HexToRGBA(self:ActiveSettings().slnamecolor)
					return r,g,b
				end,
				setFunc = function(r,g,b)
					self:ActiveSettings().slnamecolor = rltext.RGBAToHex(r, g, b, 1)
					rltext:OnFontChanged()
					if rltext.bIsPreview then
						rltext.Preview()
					end
				end,
			})


			table.insert(optionsData, {
				type = "fontblock",
				name = "Group Member Font",
				getFace = function() return self:ActiveSettings().mfont end,
				getSize = function() return self:ActiveSettings().mfontsize end,
				getOutline = function() return self:ActiveSettings().moutline end,
				getColor = function() 
					local r, g, b, a = rltext.HexToRGBA(self:ActiveSettings().mfontcolor)
					return r,g,b
				end,
				setFace = function(font)
					self:ActiveSettings().mfont = font
					rltext:OnFontChanged()
				end,
				setSize = function(size)
					self:ActiveSettings().mfontsize = size
					rltext:OnFontChanged()
				end,
				setOutline = function(outline)
					self:ActiveSettings().moutline = outline
					rltext:OnFontChanged()
				end,
				setColor = function(r, g, b)
					self:ActiveSettings().mfontcolor = rltext.RGBAToHex(r, g, b, 1)
					rltext:OnFontChanged()
					if rltext.bIsPreview then
						rltext.Preview()
					end
				end,
			})


			table.insert(optionsData, {
				type = 'colorpicker',
				name = "Member Name Color",
				tooltip = "",
				getFunc = function()
					local r, g, b, a = rltext.HexToRGBA(self:ActiveSettings().mnamecolor)
					return r,g,b
				end,
				setFunc = function(r,g,b)
					self:ActiveSettings().mnamecolor = rltext.RGBAToHex(r, g, b, 1)
					rltext:OnFontChanged()
					if rltext.bIsPreview then
						rltext.Preview()
					end
				end,
			})


			table.insert(optionsData, {
				type = 'header',
				name = "Window Settings"
			})
			--[[
			table.insert(optionsData, {
				type = 'checkbox',
				name = "Always on",
				tooltip = "",
				getFunc = function() return self:ActiveSettings().alon end,
				setFunc = function(value)
					self:ActiveSettings().alon = value 
					RLGroupControlWindow:SetHidden(false)
				end,
			})
			--]]
			table.insert(optionsData, {
				type = 'slider',
				name = "Position X",
				tooltip = "Move Left/Right",
				min = 0,
				max = math.ceil(screenWidth-self:ActiveSettings().gwinSizeX),
				getFunc = function() return self:ActiveSettings().gwinLeft end,
				setFunc = function(value)
					self:ActiveSettings().gwinLeft = value
					RLGroupControlWindow:ClearAnchors()
					RLGroupControlWindow:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, rltext:ActiveSettings().gwinLeft, rltext:ActiveSettings().gwinTop)
				end,
			})
			table.insert(optionsData, {
				type = 'slider',
				name = "Position Y",
				tooltip = "Move Up/Down",
				min = 0,
				max = math.ceil(screenHeight-self:ActiveSettings().gwinSizeY),
				getFunc = function() return self:ActiveSettings().gwinTop end,
				setFunc = function(value)
					self:ActiveSettings().gwinTop = value
					RLGroupControlWindow:ClearAnchors()
					RLGroupControlWindow:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, rltext:ActiveSettings().gwinLeft, rltext:ActiveSettings().gwinTop)
				end,
			})
			table.insert(optionsData, {
				type = 'slider',
				name = "Width",
				tooltip = "",
				min = 240,
				max = 500,
				getFunc = function() return self:ActiveSettings().gwinSizeX end,
				setFunc = function(value)
					self:ActiveSettings().gwinSizeX = value
					rltext:WinInit()
				end,
			})
			table.insert(optionsData, {
				type = 'slider',
				name = "Height",
				tooltip = "",
				min = 55,
				max = 500,
				getFunc = function() return self:ActiveSettings().gwinSizeY end,
				setFunc = function(value)
					self:ActiveSettings().gwinSizeY = value
					rltext:WinInit()
				end,
			})
			table.insert(optionsData, {
				type = 'slider',
				name = "Alpha",
				tooltip = "",
				min = 0,
				max = 100,
				getFunc = function() return self:ActiveSettings().gwinAlpha end,
				setFunc = function(value)
					self:ActiveSettings().gwinAlpha = value
					rltext:WinInit()
				end,
			})

			local additional = rltext.module:GetOptions()

			for _k,_v in pairs(additional) do
				for _,va in pairs(_v) do
					table.insert(optionsData, va)
				end
			end

			LAM:RegisterOptionControls('RaidLeadTextOptions', optionsData)
		end
	end
end
