
rltext.module = {}

rltext.module.List = {}
rltext.module.OptionList = {}
rltext.module.SavedVars = {}
rltext.wm = WINDOW_MANAGER
rltext.module.iButtons = 1


function rltext.module:LoadModules()
	for k,v in pairs(rltext.module.List) do
		if not(v.isLoaded) then
			if not(rltext:ActiveSettings().modules[v.name]) then
				rltext:ActiveSettings().modules[v.name] = {}
			end
			v.isLoaded = true
		end
		v:Init()
		v:Options()
	end
end

function rltext.module:GetOptions()
	return rltext.module.OptionList
end

function rltext.module:Get(name)
	for k,v in pairs(rltext.module.List) do
		if v.name == name then
			return v
		end
	end
	return nil
end

function rltext.module:Add(module)
	rltext.module.List[module.id] = module
end

function rltext.module:IsModuleDisabled(name)
	return rltext.module:Get(name).disabled
end

function rltext.module:GetModuleVars(name)
	return rltext:ActiveSettings().modules[name]
end

function rltext.module:GetVar(name, var)
	return rltext.module:Get(name)[var]
end

function rltext.module:SetVar(name, _var, _val)
	rltext.module:Get(name)[_var] = _val
end

local rlModule = {}
rlModule.__index = rlModule
rlModule.name = "Unknown"
rlModule.disabled = false
rlModule.isLoaded = false
rlModule.id = 999

function rlModule:Init()
end

function rlModule:Options()
end

function rlModule:IsDisabled()
	return self.disabled
end

function rlModule:GetSavedVar(var)
	return rltext:ActiveSettings().modules[self.name][var]
end

function rlModule:SaveVars()
	local rlSettings = rltext:ActiveSettings().modules

	rlSettings[self.name].disabled = self.disabled

end

function rltext.module:New()
	local ntbl = {}
	ntbl = setmetatable({}, rlModule)
	ntbl.__index = rlModule

	return ntbl
end

function rltext.module:AddOptions(mName, options)
	local noptions = {}
	table.insert(noptions, {
		type = 'header',
		name = mName
	})
	table.insert(noptions, {
		type = 'checkbox',
		name = "Disabled",
		tooltip = "",
		getFunc = function() return rltext.module:Get(mName).disabled end,
		setFunc = function(value)
			rltext.module:Get(mName).disabled = value
			rltext.module:Get(mName):SaveVars()
			rltext.module.CheckHide()
		end,
	})

	for _,va in pairs(options) do
		va.disabled = function() return rltext:ActiveSettings().modules[mName].disabled end
		table.insert(noptions, va)
	end


	rltext.module.OptionList[mName] = noptions
end

function rltext.module:AddSavedVars(mName, vars)
	rltext.module.SavedVars[mName] = vars
	local rlSettings = rltext:ActiveSettings().modules

	for _,_va in pairs(vars) do
		if type(_va) == "table" then
			if not(rlSettings[mName][_va.name]) then
				rlSettings[mName][_va.name] = _va.value
			end
		end
	end
end

function rltext.module:AddButton(mName, _button)
	rltext.module:Get(mName).button = rltext.wm:CreateControl(nil, RLGroupControlWindow, CT_BUTTON)
	rltext.module:Get(mName).button:SetDimensions(32, 32)
	rltext.module:Get(mName).button:SetAnchor(TOPLEFT, RLGroupControlWindow, TOPLEFT, 10+(30*rltext.module.iButtons), 0)

	rltext.module:Get(mName).button:SetHandler("OnClicked", _button.OnClicked)
	rltext.module:Get(mName).button:SetHandler("OnMouseEnter", function(self)
																	if _button.toolTip then
																		rltext:HeaderToolTip(self, _button.toolTip)
																	end
																	_button.OnEnter(self)
																end)
	rltext.module:Get(mName).button:SetHandler("OnMouseExit", function(self)
																	ClearTooltip(InformationTooltip) 
																	_button.OnLeave(self)
																end)

	rltext.module:Get(mName).button:SetPressedTexture("/esoui/art/buttons/plus_down.dds")
	rltext.module:Get(mName).button:SetNormalTexture("/esoui/art/buttons/plus_up.dds")
	rltext.module:Get(mName).button:SetMouseOverTexture("/esoui/art/buttons/plus_over.dds")

	rltext.module.iButtons = rltext.module.iButtons+1

	return rltext.module:Get(mName).button
end

function rltext.module.AddLabel(mName, _label)
	rltext.module:Get(mName).button = rltext.wm:CreateControl(nil, RLGroupControlWindow, CT_LABEL)
	rltext.module:Get(mName).button:SetDimensions(76, 30)
	rltext.module:Get(mName).button:SetAnchor(TOPLEFT, RLGroupControlWindow, TOPLEFT, _label.x, _label.y)

	rltext.module:Get(mName).button:SetHorizontalAlignment(0)
	rltext.module:Get(mName).button:SetVerticalAlignment(1)
	local LMP = LibStub("LibMediaProvider-1.0")
	rltext.module:Get(mName).button:SetFont(LMP:Fetch('font', _label.font).."|".._label.size.."|".._label.outline)

	rltext.module:Get(mName).button:SetText(_label.default)

	return rltext.module:Get(mName).button
end

function rltext.module.HideButtons()
	for _,_va in pairs(rltext.module.List) do
		if _va.button then
			_va.button:SetHidden(true)
		end
	end
end

function rltext.module.ShowButtons()
	for _,_va in pairs(rltext.module.List) do
		if _va.button then
			_va.button:SetHidden(false)
		end
	end
end


function rltext.module.CheckHide(force)
	if not force then force = false end

	for _,_va in pairs(rltext.module.List) do
		if _va.disabled or force then
			if _va.button then
				_va.button:SetHidden(true)
			end
		else
			if _va.button then
				_va.button:SetHidden(false)
			end
		end
	end
end

function rltext.module.Hide(force)
	if not force then force = false end

	for _,_va in pairs(rltext.module.List) do
		if force then
			if _va.button then
				_va.button:SetHidden(true)
			end
		else
			if _va.button then
				_va.button:SetHidden(false)
			end
		end
	end
end