
local raidtimer = rltext.module:New()
raidtimer.name = "RaidTimer"
raidtimer.isActive = false
raidtimer._label = nil
raidtimer._start = 0
raidtimer.id = 2


function raidtimer:Init()
	local newVars = {}
	table.insert(newVars, {
		name = "font",
		value = "Arial",
	})
	table.insert(newVars, {
		name = "size",
		value = "16",
	})
	table.insert(newVars, {
		name = "color",
		value = "FFFFFFFF",
	})
	table.insert(newVars, {
		name = "outline",
		value = "outline",
	})

	rltext.module:AddSavedVars(raidtimer.name, newVars)

	EVENT_MANAGER:RegisterForUpdate(raidtimer.name.."update", 900, raidtimer.OnUpdate)

	local nlabel = {}
	nlabel.default = ""
	nlabel.x = 30+(30*3)
	nlabel.y = 3
	nlabel.font = self:GetSavedVar("font") or "Arial"
	nlabel.size = self:GetSavedVar("size") or "16"
	nlabel.outline = self:GetSavedVar("outline") or "outline"

	rltext.module:SetVar(raidtimer.name, "_label", rltext.module.AddLabel(raidtimer.name, nlabel))
end

raidtimer.GetTimeString = function()
	local currentTime = GetTimeStamp()

	local startTime = rltext.module:GetVar(raidtimer.name, "_start")

	local totalSeconds = GetDiffBetweenTimeStamps(currentTime, startTime)

	local elapsedHours = math.floor(totalSeconds / 3600)
	local elapsedMinutes = math.floor((totalSeconds / 60) % 60)
	local elapsedSeconds = totalSeconds % 60

	return elapsedHours, elapsedMinutes, elapsedSeconds
end


function raidtimer:OnFontChanged()
	self:SaveVars()
	local font = self:GetSavedVar("font")
	local size = self:GetSavedVar("size")
	local outline = self:GetSavedVar("outline")
	local LMP = LibStub("LibMediaProvider-1.0")
	rltext.module:GetVar(self.name, "_label"):SetFont(LMP:Fetch('font', font).."|"..size.."|"..outline)
end


function raidtimer:Options()
	local optionsData = {}
	table.insert(optionsData, {
		type = "fontblock",
		name = "Font",
		getFace = function() return self:GetSavedVar("font") end,
		getSize = function() return self:GetSavedVar("size") end,
		getOutline = function() return self:GetSavedVar("outline") end,
		getColor = function() 
			local r, g, b, a = rltext.HexToRGBA(self:GetSavedVar("color"))
			return r,g,b
		end,
		setFace = function(font)
			rltext:ActiveSettings().modules[self.name]["font"] = font
			self:OnFontChanged()
		end,
		setSize = function(size)
			rltext:ActiveSettings().modules[self.name]["size"] = size
			self:OnFontChanged()
		end,
		setOutline = function(outline)
			rltext:ActiveSettings().modules[self.name]["outline"] = outline
			self:OnFontChanged()
		end,
		setColor = function(r, g, b)
			rltext:ActiveSettings().modules[self.name]["color"] = rltext.RGBAToHex(r, g, b, 1)
			self:OnFontChanged()
		end,
	})

	rltext.module:AddOptions(raidtimer.name, optionsData)
end

function raidtimer.OnStart()
	rltext.module:SetVar(raidtimer.name, "_start", GetTimeStamp())
	rltext.module:SetVar(raidtimer.name, "isActive", true)
end

function raidtimer.OnComplete(eventCode, trialName, score, totalTime)
	rltext.module:SetVar(raidtimer.name, "isActive", false)

	rltext.module:GetVar(raidtimer.name, "_label"):SetText("")

	local totalSeconds = (totalTime / 1000)
	
	local elapsedHours = math.floor(totalSeconds / 3600)
	local elapsedMinutes = math.floor((totalSeconds / 60) % 60)
	local elapsedSeconds = totalSeconds % 60

	
	local message = string.format(
		"RLE: %s completed in %d:%d:%d with a score of %d.",
		trialName,
		elapsedHours,
		elapsedMinutes,
		elapsedSeconds,
		score
	)

	d(message)
end

function raidtimer.OnFail()
	rltext.module:SetVar(raidtimer.name, "isActive", false)

	rltext.module:GetVar(raidtimer.name, "_label"):SetText("")
end

function raidtimer.OnRevive()
end


function raidtimer.OnUpdate()
	local isActive = rltext.module:GetVar(raidtimer.name, "isActive")
	local isDisabled = rltext.module:GetVar(raidtimer.name, "disabled")
	if not(isActive) then return end
	if isDisabled then
		rltext.module:GetVar(raidtimer.name, "_label"):SetText("")
		return
	end

	if not(IsRaidInProgress()) then
		rltext.module:GetVar(raidtimer.name, "_label"):SetText("")
		return
	end


	local hr, _min, _sec = raidtimer.GetTimeString()

	local raidTime = string.format(
		"%.2d:%.2d:%.2d",
		hr,
		_min,
		_sec
	)

	local color = rltext:ActiveSettings().modules[raidtimer.name]["color"]
	color = string.sub(color,1,6)

	
	rltext.module:GetVar(raidtimer.name, "_label"):SetText("|c"..color..""..raidTime)

end



EVENT_MANAGER:RegisterForEvent(raidtimer.name, EVENT_RAID_TRIAL_STARTED, raidtimer.OnStart)
EVENT_MANAGER:RegisterForEvent(raidtimer.name, EVENT_RAID_TRIAL_COMPLETE, raidtimer.OnComplete)
EVENT_MANAGER:RegisterForEvent(raidtimer.name, EVENT_RAID_TRIAL_FAILED, raidtimer.OnFail)
EVENT_MANAGER:RegisterForEvent(raidtimer.name, EVENT_RAID_REVIVE_COUNTER_UPDATE , raidtimer.OnRevive)


rltext.module:Add(raidtimer)